'***********************************************************************************
' Script Name: WMI_EventLog_CSV.vbs
'     Version: 1.0
'      Author: Jason Fossen
'Last Updated: 29.May.2004
'     Purpose: Dumps the contents of a local/remote Event Log to a local CSV file.
'              Optionally dump all logs at once.  Optionally clear one/all logs.
'              Comma-separated value (CSV) files can be opened directly in Excel or
'              imported into Access, SQL Server or any other database application;
'              a CSV file is easy to parse, compress, search, store, etc. and is 
'              therefore the preferred format for archiving Event Log data.
'       Notes: Requires RPC access to the Windows Management Instrumention (WMI)
'              service at the target Windows 2000 or later machine.  
'    Keywords: WMI, WBEM, log, event log, dump, CSV, clear, comma
'       Legal: Public Domain.  Modify and redistribute freely.  No rights reserved.
'              Script provided "AS IS" without implied warranty or guarantees. 
'***********************************************************************************
Option Explicit
On Error Resume Next




'***********************************************************************************
'  Constants, Global Variables and Common Objects.
'***********************************************************************************

Const ForAppending =      8                  'These are constants for the FileSystemObject.
Const ForOverWriting =    2
Const ForReading =        1
Const OpenAsASCII =       0     
Const OpenAsUnicode =    -1
Const OpenUsingDefault = -2

Dim oWMI                                     'The WMI connection to the remote system.
Dim oLogFile                                 'Object to represent local CSV file; append data to this.
Dim sIPaddress  : sIPaddress = "."           'IP address or name of machine to pull Event Log data from. "." means "local machine" to WMI.
Dim sFile       : sFile = "eventlog.csv"     'Local file to write Event Log data to in CSV format.
Dim sLogName    : sLogName = "System"        'Log to dump.
Dim bVerbose    : bVerbose = False           'Assume that InsertionStrings and Data aren't dumped.
Dim bDumpHex    : bDumpHex = False           'Assume that hex of Data isn't dumped.
Dim bAllLogs    : bAllLogs = False           'Assume that all logs aren't going to be dumped.
Dim bClearLogs  : bClearLogs = False         'Don't clear logs by default.
Dim bDone       : bDone = False              'Controls a wait loop for the async data collection.
Dim iStart      : iStart = Timer()           'Track how long the script runs.
Dim iCount      : iCount = 0                 'Count of event log items written to CSV file.
Dim sCleared    : sCleared = "Logs Cleared:" 'List of log names that were successfully cleared.

Dim oFileSystem : Set oFileSystem = WScript.CreateObject("Scripting.FileSystemObject")
Dim oWshShell   : Set oWshShell = WScript.CreateObject("WScript.Shell")
Dim oRegExp     : Set oRegExp = New RegExp
Dim oStdErr     : Set oStdErr = WScript.StdErr



'***********************************************************************************
' Call procedures to do the work, write some stats to StdErr. 
'***********************************************************************************

Call ProcessCommandLineArguments()
Call GetWmiConnectionToTarget()
Call GetTextFile()
Call WriteToFile()

If bClearLogs And Err.Number = 0 Then Call ClearLogs()

If Err.Number = 0 Then
    oStdErr.WriteLine "" 
    oStdErr.WriteLine "Completed successfully: " & Now()
    oStdErr.WriteLine "   CSV file written to: " & sFile
    oStdErr.WriteLine "   Run time in seconds: " & CDbl(Timer() - iStart) 
    oStdErr.WriteLine "  Lines written to CSV: " & iCount
    oStdErr.WriteLine "          " & TrimCommas(sCleared)
Else
    oStdErr.WriteLine "ERROR: Did NOT complete successfully." 
    oStdErr.WriteLine Err.Description
End If



'***********************************************************************************
'  Procedures.
'***********************************************************************************

Sub ProcessCommandLineArguments()
    On Error Resume Next
    Dim sArg

    If Not IsUsingCscript() Then
        oWshShell.Popup "Only use CSCRIPT.EXE to run this script.", 20, "Try Again Please"
        WScript.Quit
    End If


    If WScript.Arguments.Count < 3 Then
        Call ShowUsageAndQuit()
    Else
        sIPaddress = WScript.Arguments.Item(0)
        sFile = WScript.Arguments.Item(1)
        sLogName = WScript.Arguments.Item(2)  
    End If
    
    
    For Each sArg In WScript.Arguments
        Select Case UCase(sArg)
        	Case "/?", "-?", "/H", "-H", "-HELP", "--HELP" 
        	    Call ShowUsageAndQuit()
        	Case "/V", "-V", "/VERBOSE", "-VERBOSE" 
        	    bVerbose = True
            Case "/DUMPHEX", "-DUMPHEX", "/DUMP", "-DUMP", "/D", "-D"
                bDumpHex = True
                bVerbose = True  'Required for the WQL query to work, and seems logical.
            Case "/ALL", "-ALL", "ALL" 
                bAllLogs = True
            Case "/CLEAR", "-CLEAR", "CLEAR"
                bClearLogs = True
        End Select        
    Next 
    
    Call CatchAnyErrorsAndQuit("Problem processing command-line arguments.") 
End Sub



Sub GetWmiConnectionToTarget()
    On Error Resume Next
    Dim cComputerData, oItem
    
    'Don't force the user to have the "manage auditing and security log" 
    'user right (SeSecurityPrivilege) when not dumping the Security log.
    If (bAllLogs = True) Or (UCase(sLogName) = "SECURITY") Then    
        Set oWMI = GetObject("WinMgmts:{(Security)}!//" & sIPaddress & "/root/cimv2")
    Else
        Set oWMI = GetObject("WinMgmts://" & sIPaddress & "/root/cimv2")
    End If

    'The computername is needed because sometimes Microsoft writes "MACHINENAME"
    'in the computer field of log entries...which isn't very useful.  This is also
    'a good opportunity to test the WMI connection itself and fail early if necessary.   
    Set cComputerData = oWMI.ExecQuery("SELECT Name FROM Win32_ComputerSystem")
    For Each oItem In cComputerData
        sIPaddress = oItem.Name
    Next        
    
    Call CatchAnyErrorsAndQuit("Problem connecting to WMI on " & sIPaddress)    
End Sub



Sub GetTextFile()    
    On Error Resume Next
    Dim sCurrentFolder, oFile, sHeader
    
    'Keep the following header in sync with any changes to what gets dumped in WriteToFile().       
    sHeader = QuoteComma("Computer") &_         
              QuoteComma("Log") &_ 
              QuoteComma("Date") &_
              QuoteComma("Time") &_
              QuoteComma("RecordNumber") &_        
              QuoteComma("User") &_
              QuoteComma("EventCodeID") &_
              QuoteComma("SourceName") &_
              QuoteComma("EventIdentifier") &_
              QuoteComma("Category") &_
              QuoteComma("EventType") &_
              QuoteComma("Message") &_
              QuoteComma("InsertionStrings") &_
              QuoteComma("ByteDataInASCII") &_
              QuoteComma("ByteDataInHex")          
    sHeader = TrimCommas(sHeader)

    'Expand any environmental variables in the sFile path.
    If InStr(sFile, "%") <> 0 Then
        sFile = oWshShell.ExpandEnvironmentStrings(sFile)
    End If 
    
    'Assume the script's folder if the sFile's full path is not given.
    If InStr(sFile, "\") = 0 Then
        sCurrentFolder = WScript.ScriptFullName 
        sCurrentFolder = Left(sCurrentFolder, InstrRev(sCurrentFolder, "\"))
        sFile = sCurrentFolder & sFile
    End If    

    'Create the file if it doesn't exist; get it if it does.  Add headers to new files only.
    If Not oFileSystem.FileExists(sFile) Then 
        Set oLogFile = oFileSystem.CreateTextFile(sFile)
        oLogFile.WriteLine(sHeader)  'Write column headers to first line. Comment out if undesired.
    Else
        Set oFile = oFileSystem.GetFile(sFile)
        Set oLogFile = oFile.OpenAsTextStream(ForAppending, OpenUsingDefault)
    End If

    'Guarantee that insertion point is at a new line in the file.
    If oLogFile.Column <> 1 Then oLogFile.WriteLine(vbCr)   
    
    Call CatchAnyErrorsAndQuit("Problem getting CSV file.") 
End Sub



Sub WriteToFile()
    On Error Resume Next
    Dim cLogs, sList, sLine, iError, oError, iHResult, sWQL
    Dim cEventData, oItem, oAsyncContext, oSWbemSink

    If bVerbose Then
        sWQL = "SELECT * FROM Win32_NTLogEvent"
    Else
        sWQL = "SELECT Computername,Logfile,TimeGenerated,Recordnumber,User," &_
               "EventCode,SourceName,EventIdentifier,CategoryString,EventType," &_
               "Message FROM Win32_NTLogEvent"  'Everything except InsertionStrings and Data.
    End If


    If Not bAllLogs Then sWQL = sWQL & " WHERE Logfile = " & "'" & sLogName & "'" 
    

    'Asynchronous WMI queries are significantly faster than synchronous ones. An SWebmSink COM object
    'will be given to the remote system so that it can control the data handling (this is what makes
    'it asynchronous).  The object represents the local receiving end of the execution flow.
    Set oSWbemSink = WScript.CreateObject("WbemScripting.SWbemSink")
    
    'When the remote system passes data into the SWebmSink object's methods, "events" are raised on
    'the local machine which are ultimately handled by this script.  The subprocedures which handle 
    'these events must all begin with "EVENTSINK_" (or whatever is entered here) because now these
    'procedures are connected to the oSWbemSink object by that prefix name, e.g., "EVENTSINK_OnObjectReady".
    WScript.ConnectObject oSWbemSink, "EVENTSINK_"
    
    Call CatchAnyErrorsAndQuit("Problem creating WMI event sink or connecting to it.")    
    

    'Calling the ExecQuery() method returns a collection that must be Set to a variable, but calling 
    'ExecQueryAsync() will run the same WQL query and serve up the SWebmSink object for access by 
    'the remote system.  Now the script must wait for the data to come, but it doesn't do this by
    'blocking, it does it simply by not exiting and allowing the EVENTSINK_ methods to be called.
    oWMI.ExecQueryAsync oSWbemSink, sWQL 

    Call CatchAnyErrorsAndQuit("Problem executing WMI query to select data.")


    'Now prevent the script from exiting or continuing on until the remote system is done sending us
    'data.  bDone will be set to true by EVENTSINK_OnCompleted() when the remote box is finished.
    Do While Not bDone
        WScript.Sleep(1)
    Loop

    oLogFile.Close
    WScript.DisconnectObject(oSWbemSink)  'Not really necessary, but tidy. 
End Sub



Sub EVENTSINK_OnCompleted(iHResult, oError, oAsyncContext)  
    'EVENTSINK_OnCompleted() is called automatically by oSWbemSink when the WQL query is completed.
    bDone = True  'Terminates the Sleep() loop in WriteToFile().
End Sub



Sub EVENTSINK_OnObjectReady(oItem, oAsyncContext)
    'EVENTSINK_OnObjectReady() is called automatically by oSWbemSink each time a new __Event object
    'is handed off to oSWbemSink.  oItem refers to the object that raised the event to call this procedure.
    On Error Resume Next
    Dim sInsertions, sByteDataInASCII, sByteDataInHex, sLine, sComputerName

    sInsertions = ""
    sByteDataInASCII = ""
    sByteDataInHex = ""

    With oItem
        If bVerbose Then sInsertions = MakeCleanLine(JoinStringsArray(.InsertionStrings)) 'Mostly overlaps with the Message data.
        If bVerbose Then sByteDataInASCII = ByteArrayToASCII(.Data) 'Usually not needed, often overlaps with .Message and .InsertionStrings data.
        If bDumpHex Then sByteDataInHex = ByteArrayToHex(.Data)     'Very rarely needed, can produce tons of data, so watch out. 

        If .Computername = "MACHINENAME" Then
            sComputerName = sIPaddress  'Remember, sIPaddress was set to the computer's name above.
        Else
            sComputerName = .Computername
        End If

        'Keep sLine in sync with the sHeader in GetTextFile().
        sLine = QuoteComma(sComputerName) &_         
                QuoteComma(.Logfile) &_ 
                QuoteComma(GetDate(.TimeGenerated)) &_
                QuoteComma(GetTime(.TimeGenerated)) &_
                QuoteComma(.RecordNumber) &_        
                QuoteComma(.User) &_
                QuoteComma(.EventCode) &_                   
                QuoteComma(.SourceName) &_
                QuoteComma(.EventIdentifier) &_             
                QuoteComma(.CategoryString) &_
                QuoteComma(MapEventType(.EventType)) &_     
                QuoteComma(MakeCleanLine(.Message)) &_
                QuoteComma(sInsertions) &_
                QuoteComma(sByteDataInASCII) &_
                QuoteComma(sByteDataInHex)
    End With
   
    sLine = Left(sLine, Len(sLine) - 1)  'Trim off only the last comma.
    'sLine = TrimCommas(sLine)           'Use this instead if you want no trailing commas at all.
    oLogFile.WriteLine(sLine)
    
    Call CatchAnyErrorsAndQuit("Problem constructing and writing a line of data:" & vbCrLf & sLine)
    iCount = iCount + 1    'Keep track of the number of lines successfully written.
End Sub



Sub ClearLogs()
    On Error Resume Next
    Dim cLogFiles, iError, oLog
    
    If bAllLogs Then
        Set cLogFiles = oWMI.ExecQuery("SELECT * FROM Win32_NTEventLogFile")
    Else
        Set cLogFiles = oWMI.ExecQuery("SELECT * FROM Win32_NTEventLogFile WHERE LogFileName = " & "'" & sLogName & "'")
    End If

    For Each oLog In cLogFiles
        iError = oLog.ClearEventlog()
        Call CatchAnyErrorsAndQuit("Problem clearing log(s): " & iError)
        sCleared = sCleared & " " & oLog.LogFileName & ","   
    Next
End Sub



Sub CatchAnyErrorsAndQuit(sMessage)
    'Do not use "On Error Resume Next" within this procedure.
    Dim oStdErr
    If Err.Number <> 0 Then
        Set oStdErr  = WScript.StdErr  'Write to standard error stream.
        oStdErr.WriteLine vbCrLf
        oStdErr.WriteLine ">>>>>> ERROR: " & sMessage 
        oStdErr.WriteLine "Error Number: " & Err.Number 
        oStdErr.WriteLine " Description: " & Err.Description 
        oStdErr.WriteLine "Error Source: " & Err.Source  
        oStdErr.WriteLine " Script Name: " & WScript.ScriptName 
        oStdErr.WriteLine vbCrLf
        WScript.Quit Err.Number
    End If 
End Sub 



Sub ShowUsageAndQuit()
    Dim sUsage : sUsage = vbCrLf
    sUsage = sUsage & "WMI_EventLog_CSV.vbs target file.csv logname [/clear] [/v] [/dumphex] " & vbCrLf
    sUsage = sUsage & "WMI_EventLog_CSV.vbs target file.csv /all [/clear] [/v] [/dumphex] " & vbCrLf
    sUsage = sUsage & vbCrLf
    sUsage = sUsage & "    target        IP address or computername of local or remote system." & vbCrLf
    sUsage = sUsage & "    file.csv      Name or full path to local CSV file to save data." & vbCrLf
    sUsage = sUsage & "    logname       Name of the Event Log to dump." & vbCrLf
    sUsage = sUsage & "    /all          Process all of the Event Logs at target." & vbCrLf
    sUsage = sUsage & vbCrLf
    sUsage = sUsage & "    /clear        Clears log(s) after dumping." & vbCrLf
    sUsage = sUsage & "    /v            Verbose mode dumps more data." & vbCrLf
    sUsage = sUsage & "    /dumphex      Output byte array data as hex; implies verbose mode." & vbCrLf
    sUsage = sUsage & vbCrLf
    sUsage = sUsage & vbCrLf
    sUsage = sUsage & "    *** Notes: *** " & vbCrLf
    sUsage = sUsage & vbCrLf
    sUsage = sUsage & "    Purpose of the script is to dump the contents of local or remote Event" & vbCrLf
    sUsage = sUsage & "    Log(s) to a local textual CSV file for archival, search, import, etc." & vbCrLf
    sUsage = sUsage & "    Both local and remote systems should be Windows 2000 or later." & vbCrLf
    sUsage = sUsage & vbCrLf
    sUsage = sUsage & "    If logname contains a space character, enclose name in double-quotes. If" & vbCrLf
    sUsage = sUsage & "    you don't use the /all switch and you specify a logname that doesn't exist" & vbCrLf
    sUsage = sUsage & "    at the target, you will NOT get an error message (and no data will be" & vbCrLf
    sUsage = sUsage & "    written to the CSV file).  The logname is not case sensitive." & vbCrLf
    sUsage = sUsage & vbCrLf
    sUsage = sUsage & "    file.csv may be a simple filename or a full local path.  Simple filename" & vbCrLf
    sUsage = sUsage & "    will use the script's working folder.  File will be created if it does" & vbCrLf
    sUsage = sUsage & "    not already exist.  Path may include environmental variables.  Column" & vbCrLf
    sUsage = sUsage & "    header is added only when the script creates the file, hence, pre-create" & vbCrLf
    sUsage = sUsage & "    an empty file if you don't want a first row added with column names." & vbCrLf
    sUsage = sUsage & "    Appending data to an existing file does not write the headers again." & vbCrLf
    sUsage = sUsage & vbCrLf
    sUsage = sUsage & "    Verbose mode includes insertion strings and byte array data in ASCII." & vbCrLf
    sUsage = sUsage & "    Byte data is filtered to exclude carriage returns, nulls, and line feeds." & vbCrLf
    sUsage = sUsage & "    Verbose mode is rarely necessary because of the redundancy of the data." & vbCrLf
    sUsage = sUsage & vbCrLf
    sUsage = sUsage & "    Dumphex implies verbose mode and dumps unfiltered byte array data in hex." & vbCrLf
    sUsage = sUsage & "    Dumphex is VERY rarely necessary and increases CSV file size substantially." & vbCrLf

    WScript.Echo sUsage
    WScript.Quit(Err.Number)
End Sub



'***********************************************************************************
'  Functions.
'***********************************************************************************


Function IsUsingCscript()
    Dim iPosition
    iPosition = InStr( LCase(WScript.FullName) , "cscript.exe" )
    If iPosition = 0 Then IsUsingCscript = False Else IsUsingCscript = True 
End Function



Function TrimCommas(sData)
    TrimCommas = RTrim(sData)
    Dim iLength : iLength = Len(sData)

    If iLength = 0 Then
        TrimCommas = ""
        Exit Function
    End If

    If InStrRev(sData, ",,,") = (iLength - 2) Then
        TrimCommas = Left(TrimCommas, iLength - 3)
    ElseIf InStrRev(sData, ",,") = (iLength - 1) Then
        TrimCommas = Left(TrimCommas, iLength - 2)
    ElseIf InStrRev(sData, ",") = iLength Then
        TrimCommas = Left(TrimCommas, iLength - 1)    
    End If
    
    iLength = Len(TrimCommas)
    If InStrRev(TrimCommas, ",") = iLength Then 
        TrimCommas = TrimCommas(TrimCommas)      'Recursive.
    End If
End Function



Function MapEventType(iNum)
    Select Case CInt(iNum)
    	Case 0 : MapEventType = "<Success>"    
    	Case 1 : MapEventType = "<Error>"
    	Case 2 : MapEventType = "<Warning>"
        Case 3 : MapEventType = "<Information>"
    	Case 4 : MapEventType = "<Audit-Success>"
    	Case 5 : MapEventType = "<Audit-Failure>"
    	Case Else : MapEventType = "<???>"  'Should never get here.
    End Select
End Function



Function JoinStringsArray(ByRef aArray)
    If VarType(aArray) = vbNull Then
        JoinStringsArray = ""
        Exit Function
    Else
        JoinStringsArray = Join(aArray)
        JoinStringsArray = Replace(JoinStringsArray, "(NULL)", "")
    End If
End Function



Function MakeCleanLine(sText)
    If (VarType(sText) = vbNull) Or (sText = "") Or (Len(sText) = 0) Then
        MakeCleanLine = ""
        Exit Function
    Else
        If Not IsObject(oRegExp) Then Set oRegExp = New RegExp
        oRegExp.Global = True
        oRegExp.MultiLine = True  
        oRegExp.Pattern = "\s+"     'One or more whitespaces.   
        MakeCleanLine = oRegExp.Replace(sText," ")
        MakeCleanLine = Trim(MakeCleanLine)
    End If
End Function



Function ByteArrayToHex(ByRef aArray)
    Dim i
    If VarType(aArray) = vbNull Then
        ByteArrayToHex = ""
        Exit Function
    Else
        For i = 0 To UBound(aArray)
            ByteArrayToHex = ByteArrayToHex & Right("0" & Hex(aArray(i)), 2) & " "
        Next
    End If
    ByteArrayToHex = RTrim(ByteArrayToHex)  'Trim off trailing space character.
End Function



Function ByteArrayToASCII(ByRef aArray)
    Dim i, iNum
    If VarType(aArray) = vbNull Then
        ByteArrayToASCII = ""
        Exit Function
    Else
        For i = 0 To UBound(aArray)
            iNum = CInt(aArray(i))
            If (iNum <> 0) And (iNum <> 10) and (iNum <> 13) Then '0=Null, 10=LF, 13=CR.
                ByteArrayToASCII = ByteArrayToASCII & Chr(iNum)
            End If
        Next
    End If
End Function



Function QuoteComma(sData)
    If (sData = "") Or (VarType(sData) = vbNull) Or (Len(sData) = 0) Then
        QuoteComma = ","                   
        Exit Function
    End If

    QuoteComma = CStr(sData)    
    QuoteComma = Replace(QuoteComma, """", "``")  'Replace double-quotes with two backticks.
    QuoteComma = Replace(QuoteComma, vbCrLf, "")  'Delete the weird extra newlines MS adds for no sane reason...
    QuoteComma = Trim(QuoteComma)                 '    ...and better to do it here than with a RegExp later (expensive)
    
    If InStr(QuoteComma, ",") = 0 Then
        QuoteComma = QuoteComma & ","
    Else
        QuoteComma = """" & QuoteComma & """" & ","  'Double-quote the string because of the comma(s) inside it.
    End If
End Function



Function GetTime(sWmiDate)
    'Pass in a WMI date like "20041229114458.000000-360"
    'Returns in military hour:min:sec format, with hours from 0 to 23.
    'Does NOT set to UTC or Zulu time; this is local time at the machine.
    GetTime = Mid(sWmiDate,9,2) & ":" & Mid(sWmiDate,11,2) & ":" & Mid(sWmiDate,13,2)    
End Function



Function GetDate(sWmiDate)
    'Pass in a WMI date like "20041229114458.000000-360"    
    Dim sYear, sMonth, sDay
    sYear =  Mid(sWmiDate,3,2)
    sMonth = Mid(sWmiDate,5,2)
    sDay =   Mid(sWmiDate,7,2)
        
    Select Case CInt(sMonth)
        Case 1  : sMonth = "Jan"
        Case 2  : sMonth = "Feb"
        Case 3  : sMonth = "Mar"
        Case 4  : sMonth = "Apr"
        Case 5  : sMonth = "May"
        Case 6  : sMonth = "Jun"
        Case 7  : sMonth = "Jul"
        Case 8  : sMonth = "Aug"
        Case 9  : sMonth = "Sep"
        Case 10 : sMonth = "Oct"
        Case 11 : sMonth = "Nov"
        Case 12 : sMonth = "Dec"
    	Case Else : sMonth = "???"  'Should never get here.        
    End Select
    
    GetDate = sDay & "-" & sMonth & "-" & sYear  'E.g., "25-May-04" local, not UTC.
End Function


'END OF SCRIPT************************************************************************
