'*****************************************************
' Script Name: IIS_Hacking_Attempts.vbs
'     Version: 2.2
'      Author: Jason Fossen
'Last Updated: 3/14/03
'     Purpose: Search IIS logs for hacking attempts.
'       Usage: Pass one or more IIS logs to the script as arguments.
'       Notes: Install the latest version of WSH to ensure that the
'              regular expressions work correctly.  You must do this
'              if the script always reports "0 hacking attempts".  Get
'              the WSH from http://msdn.microsoft.com/scripting/
'    Keywords: IIS, logs, HTTP, IDS, hack, hacking, signatures
'       Legal: Public Domain.  Modify and redistribute freely.  No rights reserved.
'              Script provided "as is" without implied warranty or guarantees.  Use
'              at your own risk and only on networks with prior written permission.          
'*****************************************************

On Error Resume Next

'*********************************************************************
'Declare constants, variables, common objects, etc..
'*********************************************************************
Const ForReading = 1 
Dim aLogFiles()     'Resizable array with the list of IIS log files.
Dim sResult         'This will hold the summary of results to be printed out.
Dim sBadLogLines    'This will hold all the detected suspicious log entries.
Set oFileSystem = WScript.CreateObject("Scripting.FileSystemObject")
sCurrentFolder = GetCurrentFolder()


'*********************************************************************
'Create an array of attack signatures: aSignatures.
'Each attack signature has three parts: the regular expression to match, a description of the exploit
'being attempted, and a counter which starts at zero.  Add more signatures as desired.
'*********************************************************************
Dim aSignatures(24,3) 'Set equal to number of signatures as new ones are added (or write a procedure to "ReDim Preserve" it).

aSignatures(0,0) = "msadcs\.dll"                                            'Regular expression pattern.
aSignatures(0,1) = " attempts to access msadcs.dll (RDS exploit)."          'Description of hack.
aSignatures(0,2) = 0                                                        'Counter.

aSignatures(1,0) = "::\$DATA"
aSignatures(1,1) = " ::$DATA exploit attempts."
aSignatures(1,2) = 0

aSignatures(2,0) = "\.asp\."
aSignatures(2,1) = " .asp. exploit attempts."
aSignatures(2,2) = 0

aSignatures(3,0) = "sam\._ "
aSignatures(3,1) = " attempts to download the SAM backup file."
aSignatures(3,2) = 0

aSignatures(4,0) = "\.ida |\.idq |\.htw "
aSignatures(4,1) = " attempts to access an Index Server page."
aSignatures(4,2) = 0

aSignatures(5,0) = "/iissamples"
aSignatures(5,1) = " attempts to access an IIS sample page."
aSignatures(5,2) = 0

aSignatures(6,0) = "showcode\.asp"
aSignatures(6,1) = " attempts to access the SHOWCODE.ASP page."
aSignatures(6,2) = 0

aSignatures(7,0) = "/certsrv"
aSignatures(7,1) = " attempts to access Certificate Server."
aSignatures(7,2) = 0

aSignatures(8,0) = "\.asp\.htr"
aSignatures(8,1) = " attempts to append .HTR to an ASP page."
aSignatures(8,2) = 0

aSignatures(9,0) = "iisadmpwd"
aSignatures(9,1) = " attempts to access the IIS 4.0 change password scripts."
aSignatures(9,2) = 0

'Note: It is assumed that you have renamed the Administrator account of course.
aSignatures(10,0) = ".+\\(?:sys)*admin.* .|.+\\root ."
aSignatures(10,1) = " attempts to log on with an administrative account."
aSignatures(10,2) = 0

aSignatures(11,0) = "GET /default\.ida"
aSignatures(11,1) = " probes by the Code Red Worm."
aSignatures(11,2) = 0

aSignatures(12,0) = "global\.asa"
aSignatures(12,1) = " attempts to access the GLOBAL.ASA file."
aSignatures(12,2) = 0

aSignatures(13,0) = "\\winnt|/winnt"
aSignatures(13,1) = " attempts to access \Winnt or its subfolders."
aSignatures(13,2) = 0

aSignatures(14,0) = "cmd\.exe"
aSignatures(14,1) = " attempts to access CMD.EXE."
aSignatures(14,2) = 0

aSignatures(15,0) = "/printers"
aSignatures(15,1) = " attempts to access the /printers folder."
aSignatures(15,2) = 0

aSignatures(16,0) = "\.\.\\"   'Notice it is a backslash, not forward.
aSignatures(16,1) = " attempts to use ..\ for directory traversal."
aSignatures(16,2) = 0

aSignatures(17,0) = "/iisadmin"
aSignatures(17,1) = " attempts to access the Administration Website."
aSignatures(17,2) = 0

aSignatures(18,0) = "/system32/cmd.+/c\+tftp"
aSignatures(18,1) = " attempts to execute TFTP.EXE locally."
aSignatures(18,2) = 0

aSignatures(19,0) = "/\./"     ' "/./" is equivalent to "/" when interpreted by IIS.
aSignatures(19,1) = " attempts to use /./ to obscure the URL pattern."
aSignatures(19,2) = 0

aSignatures(20,0) = "(?:GET|POST|PUT).+\\"     
aSignatures(20,1) = " uses of a backslash in a URL."
aSignatures(20,2) = 0

aSignatures(21,0) = "GET /favicon\.ico "     
aSignatures(21,1) = " times someone bookmarked your site with IE 5.0+."   'Not a hacking signature of course.
aSignatures(21,2) = 0

aSignatures(22,0) = "GET /exchange"     
aSignatures(22,1) = " attempts to access Exchange Outlook Web Access site."   
aSignatures(22,2) = 0

aSignatures(23,0) = "GET /scripts/|GET /cgi-bin/|GET /cgibin/"     
aSignatures(23,1) = " attempts to access a /scripts or /cgi-bin folder."   'Shouldn't have these folders!   
aSignatures(23,2) = 0



'*********************************************************************
'Call procedures which do the work of the script.
'*********************************************************************
Call CreateFileList()
Call SearchEachFile()
Call GenerateReport()



'*********************************************************************
'Procedure: CreateFileList()
'  Purpose: Create an array (aLogFiles) of the IIS log files to be searched.
'*********************************************************************
Sub CreateFileList
    If WScript.Arguments.Count = 0 Then
        Call ShowUsage()
    Else
       ReDim aLogFiles(WScript.Arguments.Count)
    End If
        
    i = 0
    For Each sArg In WScript.Arguments
        If InStr(sArg, "\") = 0 Then sArg = sCurrentFolder & sArg   'Dropped and Send-To file args have full path already, but command-lines don't.
        If Not oFileSystem.FileExists(sArg) Then 
            WScript.Echo "File " & sArg & " not found!"
            WScript.Quit 
        End If 
        aLogFiles(i) = sArg
        i = i + 1
    Next 
End Sub 'CreateFileList


'*********************************************************************
'Procedure: SearchEachFile()
'  Purpose: Searches each log file for hack attempts.
'*********************************************************************
Sub SearchEachFile
    Set oRegExp = New RegExp                             
    oRegExp.IgnoreCase = True                            
    oRegExp.Global = True                                
    
    For i = 0 to UBound(aLogFiles) - 1 
        Set oFile = oFileSystem.OpenTextFile(aLogFiles(i),ForReading)
        Do While Not oFile.AtEndOfStream
            sLine = oFile.ReadLine
            bFoundBadLine = False

            oRegExp.Pattern = "^\x00{30}"        'IIS pads the end of new log files with hundreds of Null characters, which, without this test, will each be read as a single line and searched.
            If Not oRegExp.Test(sLine) Then

                For x = 0 to UBound(aSignatures) - 1 
                    oRegExp.Pattern = aSignatures(x,0)

                    If oRegExp.Test(sLine) Then 
                        aSignatures(x,2) = aSignatures(x,2) + 1
                        bFoundBadLine = True
                    End If                    
                Next    'next signature
            End If      'test for null character padding

            If bFoundBadLine Then sBadLogLines = sBadLogLines & sLine & vbCrLf

        Loop            'to next line in file
        oFile.Close 
    Next                'next file in aLogFiles
End Sub


'*********************************************************************
'Procedure: GenerateReport()
'  Purpose: Output a summary of log file analysis.
'*********************************************************************
Sub GenerateReport
    sResult =           "--------------------------------------------------" & vbCrLf
    For x = 0 to UBound(aLogFiles) - 1
        sResult = sResult & "Log File: " & aLogFiles(x) & vbCrLf
    Next
    sResult = sResult & "--------------------------------------------------" & vbCrLf

    bDetectedHacks = False

    For i = 0 to UBound(aSignatures) - 1
        If aSignatures(i,2) > 0 Then
            bDetectedHacks = True
            sResult = sResult & aSignatures(i,2) & aSignatures(i,1) & vbCrLf 
        End If 
    Next
    
    If Not bDetectedHacks Then sResult = sResult & "0 hacking attempts detected." & vbCrLf
    WScript.Echo sResult
    
    WScript.Echo vbCrLf & sBadLogLines   'Comment out this line to suppress the showing of each log entry.
End Sub


'*****************************************************
' Helper functions and procedures.
'*****************************************************

Function GetCurrentFolder()
    strFN = WScript.ScriptFullName
    GetCurrentFolder = Left(strFN, InstrRev(strFN, "\"))
End Function 

Sub ShowUsage()
    WScript.Echo "This script will summarize the hacking attempts in " &_
                 "one or more IIS log files.  Enter one or more script "&_
                 "files as arguments at the command line, or drag and "&_
                 "drop the log file(s) onto the script icon itself."
    WScript.Quit 
End Sub


'END OF SCRIPT ***************************************
